# Copyright (C) 2010 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


from bzrlib import (
    branch as _mod_branch,
    commands,
    errors,
    option,
    trace,
)
from bzrlib.plugins.launchpad import (
    lp_api,
    lp_registration,
)


class cmd_lp_link_bug(commands.Command):
    """Link a branch to a Launchpad bug.

    The bug_id supplied will be used to look up a bug and link it to the
    branch.  If the branch has not been pushed to Launchpad, this command
    pushes it first.
    """

    takes_args = ['bug_id']
    takes_options = [
        option.Option('assign', help='Assign the bug to the branch owner.'),
        option.Option('in-progress', help='Mark the bug as in-progress.'),
        ]

    def run(self, bug_id, assign=False, in_progress=False):
        service = lp_registration.LaunchpadService(lp_instance='production')
        launchpad = lp_api.login(service)
        branch = lp_api.LaunchpadBranch.from_bzr(
            launchpad, _mod_branch.Branch.open_containing('.')[0])
        try:
            bug = launchpad.bugs[bug_id]
        except KeyError:
            raise errors.BzrCommandError('No bug #%s on Launchpad.' % bug_id)
        if assign or in_progress:
            for task in bug.bug_tasks:
                if task.target == branch.lp.project:
                    break
            else:
                raise errors.BzrCommandError(
                    'No bug task for %s' % branch.lp.project)
            if assign:
                task.assignee = branch.lp.owner
            if in_progress:
                task.status = 'In Progress'
            task.lp_save()
        message = 'Linked to bug #%d: %s' % (bug.id, bug.title)
        branch.lp.linkBug(bug=bug)
        trace.note(message)
