#!/usr/bin/python2.5
#
# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import re
import sys

from bzrlib import commands, trace, urlutils
from bzrlib.branch import Branch
from bzrlib.config import LocationConfig
from bzrlib.errors import NoSuchRevision
from bzrlib.transport import get_transport


QASTAGING_ROOT='https://qastaging.launchpad.net/'
PRODUCTION_ROOT='https://launchpad.net/'


def get_staging_revision():
    """Get the revision of db-stable deployed on staging.

    :return: The staging revno as an int. Corresponds to a revision of
        lp:launchpad/db-stable.
    """
    t = get_transport('https://staging.launchpad.net/')
    last_line = t.get_bytes('successful-updates.txt').splitlines()[-1]
    return int(last_line.split()[-1])


def get_deployed_revno(root_url):
    """Get the revision of stable deployed on a launchpad instance.

    :param root_url: The URL of the root site for the desired instance.  e.g.
        QASTAGING_ROOT.
    :return: The deployed revno as an int. For qastaging and production, this
    corresponds to a revision of lp:launchpad/stable.
    """
    t = get_transport(root_url)
    html = t.get_bytes('index.html')
    revision_re = re.compile(r' +r(\d+)')
    for line in html.splitlines():
        matches = revision_re.search(line)
        if matches:
            return int(matches.group(1))
    raise ValueError("Could not find revision number on %s" % root_url)


def update_local_branch(local, remote, rev_id):
    """Ensure the supplied revision is present in the local branch."""
    local_url = urlutils.local_path_from_url(local.base)
    if not local.repository.has_revision(rev_id):
        trace.note("Updating local mirror %s" % local_url)
        local.repository.fetch(remote.repository, rev_id)
    assert local.repository.has_revision(rev_id)


def get_rev_id(deployed_branch, deployed_revno, remote_location):
    """Return the revid of the supplied revno, and fetch if not local."""
    try:
        deployed_rev_id = deployed_branch.get_rev_id(deployed_revno)
    except NoSuchRevision:
        if remote_location is None:
            raise
        remote = Branch.open(remote_location)
        remote.lock_read()
        try:
            deployed_rev_id = remote.get_rev_id(deployed_revno)
            update_local_branch(deployed_branch, remote, deployed_rev_id)
        finally:
            remote.unlock()
    return deployed_rev_id


def is_present(local_branch, deployed_location, deployed_revno):
    local_mirror = LocationConfig(deployed_location).get_user_option(
        'local_location')
    if local_mirror is None:
        trace.note(
            'Please configure a local_location for %s in locations.conf '
            'to improve performance.\n' % deployed_location)
        deployed_branch = Branch.open(deployed_location)
        remote_location = None
    else:
        deployed_branch = Branch.open(local_mirror)
        remote_location = deployed_location
    deployed_branch.lock_write()
    try:
        deployed_rev_id = get_rev_id(deployed_branch, deployed_revno,
                                     remote_location)
        graph = deployed_branch.repository.get_graph(local_branch.repository)
        return graph.is_ancestor(local_branch.last_revision(), deployed_rev_id)
    finally:
        deployed_branch.unlock()


stable = 'bzr+ssh://bazaar.launchpad.net/~launchpad-pqm/launchpad/stable'
dbstable = 'bzr+ssh://bazaar.launchpad.net/~launchpad-pqm/launchpad/db-stable'
def main(argv):
    if len(sys.argv) > 1:
        location = sys.argv[1]
    else:
        location = '.'


class cmd_qa_ready(commands.Command):
    """Determine whether a Launchpad branch is ready for QA.

    It will report where the branch's tip revision has been deployed:
    qastaging, staging, or neither.

    To speed up these determinations, local mirrors of qastaging and staging
    can be configured.  To do this, set the 'local_location' values
    for bzr+ssh://bazaar.launchpad.net/~launchpad-pqm/launchpad/stable and
    bzr+ssh://bazaar.launchpad.net/~launchpad-pqm/launchpad/db-stable in
    locations.conf.
    """

    takes_args = ['location?']

    def run(self, location='.'):
        b = Branch.open_containing(location)[0]
        b.lock_read()
        try:
            trace.note('Branch: %s' % b.base)
            self.outf.write('Deployed on production: %s\n' % is_present(
                b, stable, get_deployed_revno(PRODUCTION_ROOT)))
            self.outf.write('Deployed on qastaging: %s\n' % is_present(
                b, stable, get_deployed_revno(QASTAGING_ROOT)))
            self.outf.write('Deployed on staging: %s\n' % is_present(
                b, dbstable, get_staging_revision()))
        finally:
            b.unlock()
